package	jp.co.sds_corp.util.config;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
* IConfigurationの内、デフォルトの実装を提供します。
* 現在設定ファイルがXML形式を前提としているため、この抽象化に
* 意味はありません。
*/
public abstract class AbstractConfiguration implements IConfiguration
{
	private static final long serialVersionUID = 1L;

	public IConfiguration getParent() {
		throw new UnsupportedOperationException();
	}

	public IConfiguration getChild( String name ) {
		throw new UnsupportedOperationException();
	}

	public Iterator<IConfiguration> getChilds() {
		throw new UnsupportedOperationException();
	}

	public List<IConfiguration> listChild() {
		List<IConfiguration> ret = new ArrayList<IConfiguration>();
		Iterator<IConfiguration> itr = getChilds();
		while ( itr.hasNext()) {
			ret.add( itr.next() );
		}
		return ret;
	}

	public Iterator<IConfiguration> getChilds( String name ) {
		throw new UnsupportedOperationException();
	}

	public List<IConfiguration> listChild( String name ) {
		List<IConfiguration> ret = new ArrayList<IConfiguration>();
		Iterator<IConfiguration> itr = getChilds( name );
		while ( itr.hasNext()) {
			ret.add( itr.next() );
		}
		return ret;
	}

	public IConfiguration getChild( String name, String attrName, String attrValue ) {
		IConfiguration ret = null;
		Iterator<IConfiguration> itr = getChilds( name );
		while (itr.hasNext()) {
			IConfiguration config = itr.next();
			String attr = config.getAttributes().get(attrName, null);
			if (attr == null) {
				continue;
			}
			if (attr.equals(attrValue)) {
				ret = config;
				break;
			}
		}
		return ret;
	}

	public IConfiguration getAttributes() {
		throw new UnsupportedOperationException();
	}

	public String getName() {
		throw new UnsupportedOperationException();
	}

	public IConfiguration addChild( IConfiguration chlid ) {
		throw new UnsupportedOperationException();
	}
	public IConfiguration addChild( String key ) {
		throw new UnsupportedOperationException();
	}
	public IConfiguration addChild( String key, String value ) {
		throw new UnsupportedOperationException();
	}
	public void set( String key ) {
		throw new UnsupportedOperationException();
	}
	public void setAttribute( String key, String value ) {
		throw new UnsupportedOperationException();
	}

	/** */
	public int getInteger( String key )
		throws UnspecifiedPropertyException,
				ConfigurationFormatException
	{
		String value = get( key );
		try {
			return Integer.parseInt( value );
		} catch ( NumberFormatException ex ) {
			throw new ConfigurationFormatException( key, value );
		}
	}

	public int getInteger( String key, int defaultValue )
		throws ConfigurationFormatException
	{
		try {
			return getInteger( key );
		} catch ( UnspecifiedPropertyException ex ) {
			return defaultValue;
		}
	}

	/** */
	public float getFloat( String key )
		throws UnspecifiedPropertyException,
				ConfigurationFormatException
	{
		String value = get( key );
		try {
			return Float.parseFloat( value );
		} catch ( NumberFormatException ex ) {
			throw new ConfigurationFormatException( key, value );
		}
	}

	public float getFloat( String key, float defaultValue )
		throws ConfigurationFormatException
	{
		try {
			return getFloat( key );
		} catch ( UnspecifiedPropertyException ex ) {
			return defaultValue;
		}
	}

	/** */
	public boolean getBoolean( String key )
		throws UnspecifiedPropertyException
	{
		return Boolean.valueOf( get( key )).booleanValue();
	}

	public boolean getBoolean( String key, boolean defaultValue ) {
		try {
			return getBoolean( key );
		} catch ( UnspecifiedPropertyException ex ) {
			return defaultValue;
		}
	}

	/** */
	public String get( String key, String defaultValue ) {
		try {
			return get( key );
		} catch ( UnspecifiedPropertyException ex ) {
			return defaultValue;
		}
	}
}
