package jp.co.sds_corp.util.config;

import java.util.List;
import java.util.Vector;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.UserDataHandler;

/**
 * odeクラス
 */
public class ConfigurationNode implements Node
{
	Node node;
	ConfigurationNode	parent;
	ConfigurationNode	prev;
	ConfigurationNode	next;
	List<Node>		listChild;
	NodeListImpl	childList;

	public ConfigurationNode() {
	}

	public ConfigurationNode( Node node, ConfigurationNode parent )
	{
		this.node = node;
		this.parent = parent;
		copyChild();
	}

	protected void copyChild() {
		listChild = new Vector<Node>();
		childList = new NodeListImpl( listChild );

		ConfigurationNode	prev = null;
		NodeList list = node.getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			ConfigurationNode node = null;
			switch ( child.getNodeType()) {
			case Node.ELEMENT_NODE:
				node = new ConfigurationElement((Element )child, this );
				break;
			default:
				node = new ConfigurationNode( child, this );
				break;
			}
			listChild.add( node );
			if ( prev != null ) {
				node.setPrevSibling( prev );
				prev.setNextSibling( node );
			}
			prev = node;
		}
	}

	protected void setPrevSibling( ConfigurationNode prev ) {
		this.prev = prev;
	}

	protected void setNextSibling( ConfigurationNode next ) {
		this.next = next;
	}

	public ConfigurationElement getChildElement( String name ) {
		ConfigurationElement ret = null;
		NodeList list = getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE &&
				child.getNodeName().equals( name ))
			{
				ret = (ConfigurationElement )child;
				break;
			}
		}
		return ret;
	}

	public List<ConfigurationElement> listChildElement() {
		List<ConfigurationElement> ret = new Vector<ConfigurationElement>();
		NodeList list = getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE ) {
				ret.add( (ConfigurationElement )child );
			}
		}
		return ret;
	}

	public List<ConfigurationElement> listChildElement( String name ) {
		List<ConfigurationElement> ret = new Vector<ConfigurationElement>();
		NodeList list = getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			if ( child.getNodeType() == Node.ELEMENT_NODE &&
				child.getNodeName().equals( name ))
			{
				ret.add( (ConfigurationElement )child );
			}
		}
		return ret;
	}

	public String getText() {
		String ret = null;
		NodeList list = getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			if ( child.getNodeType() == Node.TEXT_NODE ) {
				ret = child.getNodeValue();
				break;
			}
		}
		return ret;
	}

	public String getTypeName() {
		String ret = "";
		switch ( getNodeType() ) {
		case Node.ELEMENT_NODE :        ret = "ELEMENT"; break;
		case Node.ATTRIBUTE_NODE :      ret = "ATTRIBUTE"; break;
		case Node.TEXT_NODE :           ret = "TEXT"; break;
		case Node.CDATA_SECTION_NODE :  ret = "CDATA_SECTION"; break;
		case Node.ENTITY_REFERENCE_NODE : ret = "ENTITY_REFERENCE"; break;
		case Node.ENTITY_NODE :         ret = "ENTITY"; break;
		case Node.PROCESSING_INSTRUCTION_NODE :
			ret = "PROCESSING_INSTRUCTION"; break;
		case Node.COMMENT_NODE :        ret = "COMMENT"; break;
		case Node.DOCUMENT_NODE :       ret = "DOCUMENT"; break;
		case Node.DOCUMENT_TYPE_NODE :  ret = "DOCUMENT_TYPE"; break;
		case Node.DOCUMENT_FRAGMENT_NODE : ret = "DOCUMENT_FRAGMENT"; break;
		case Node.NOTATION_NODE :       ret = "NOTATION"; break;
		default: ret = "UNKNOWN"; break;
		}
		return ret;
	}

	public String printTree( int depth ) {
		StringBuilder ret = new StringBuilder();
		ret.append( depth + ")" + toString());
		NodeList list = getChildNodes();
		for ( int i = 0; i < list.getLength(); i++ ) {
			Node child = list.item( i );
			ret.append( "\n" );
			ret.append( ((ConfigurationNode )child).printTree( depth + 1 ));
		}
		return ret.toString();
	}

	public String printElementTree( int depth ) {
		StringBuilder ret = new StringBuilder();
		ret.append( depth + ")" + toString() + "[" + getText() + "]" );
		List<ConfigurationElement> list = listChildElement();
		for ( int i = 0; i < list.size(); i++ ) {
			ConfigurationElement child = list.get( i );
			ret.append( "\n" );
			ret.append( child.printElementTree( depth + 1 ));
		}
		return ret.toString();
	}

	@Override
	public String toString() {
		StringBuilder ret = new StringBuilder();
		ret .append( getTypeName() + ","
			+ " name=" + node.getNodeName()
			+ " value=[" + node.getNodeValue() + "]" );
		NamedNodeMap map = node.getAttributes();
		if ( map != null ) {
			ret.append( " attr[" );
			for ( int i = 0; i < map.getLength(); i++ ) {
				Node attr = map.item( i );
				ret.append( " " + attr.getNodeName() + "=" + attr.getNodeValue());
			}
			ret.append( "]" );
		}
		return ret.toString();
	}

	@Override
	public Node appendChild(Node newChild) {
		return node.appendChild( newChild );
	}
	@Override
	public Node cloneNode(boolean deep) {
		return node.cloneNode( deep );
	}
	@Override
	public short compareDocumentPosition(Node other) {
		return node.compareDocumentPosition( other );
	}
	@Override
	public NamedNodeMap getAttributes() {
		return node.getAttributes();
	}
	@Override
	public String getBaseURI() {
		return node.getBaseURI();
	}
	@Override
	public NodeList getChildNodes() {
		return childList;
	}
	@Override
	public Object getFeature(String feature, String version) {
		return node.getFeature( feature, version );
	}
	@Override
	public Node getFirstChild() {
		if ( listChild.size() == 0 ) {
			return null;
		}
		return listChild.get( 0 );
	}
	@Override
	public Node getLastChild() {
		if ( listChild.size() == 0 ) {
			return null;
		}
		return listChild.get( listChild.size() - 1 );
	}
	@Override
	public String getLocalName() {
		return node.getLocalName();
	}
	@Override
	public String getNamespaceURI() {
		return node.getNamespaceURI();
	}
	@Override
	public Node getNextSibling() {
		return next;
	}
	@Override
	public String getNodeName() {
		return node.getNodeName();
	}
	@Override
	public short getNodeType() {
		return node.getNodeType();
	}
	@Override
	public String getNodeValue() {
		return node.getNodeValue();
	}
	@Override
	public Document getOwnerDocument() {
		return node.getOwnerDocument();
	}
	@Override
	public Node getParentNode() {
		return parent;
	}
	@Override
	public String getPrefix() {
		return node.getPrefix();
	}
	@Override
	public Node getPreviousSibling() {
		return prev;
	}
	@Override
	public String getTextContent() {
		return node.getTextContent();
	}
	@Override
	public Object getUserData(String key) {
		return node.getUserData( key );
	}
	@Override
	public boolean hasAttributes() {
		return node.hasAttributes();
	}
	@Override
	public boolean hasChildNodes() {
		return ( listChild.size() > 0 );
	}
	@Override
	public Node insertBefore(Node newChild, Node refChild) {
		return node.insertBefore( newChild, refChild );
	}
	@Override
	public boolean isDefaultNamespace(String namespaceURI) {
		return node.isDefaultNamespace( namespaceURI );
	}
	@Override
	public boolean isEqualNode(Node arg) {
		return node.isEqualNode( arg );
	}
	@Override
	public boolean isSameNode(Node other) {
		return node.isSameNode( other );
	}
	@Override
	public boolean isSupported(String feature, String version) {
		return node.isSupported( feature, version );
	}
	@Override
	public String lookupNamespaceURI(String prefix) {
		return node.lookupNamespaceURI( prefix );
	}
	@Override
	public String lookupPrefix(String namespaceURI) {
		return node.lookupPrefix( namespaceURI );
	}
	@Override
	public void normalize() {
		node.normalize();
	}
	@Override
	public Node removeChild(Node oldChild) {
		return node.removeChild( oldChild );
	}
	@Override
	public Node replaceChild(Node newChild, Node oldChild) {
		return node.replaceChild( newChild, oldChild );
	}
	@Override
	public void setNodeValue(String nodeValue) {
		node.setNodeValue( nodeValue );
	}
	@Override
	public void setPrefix(String prefix) {
		node.setPrefix( prefix );
	}
	@Override
	public void setTextContent(String textContent) {
		node.setTextContent( textContent );
	}
	@Override
	public Object setUserData(String key, Object data, UserDataHandler handler) {
		return node.setUserData( key, data, handler );
	}

}
