package jp.co.sds_corp.util.crypt;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.spec.KeySpec;

import jp.co.sds_corp.util.config.IConfiguration;

public abstract class AbstractStoreCipher extends AbstractCipher implements IStoreCipher
{
	String storeType = "JKS";
	String storePassword;
	String alias;
	KeyStore store = null;

	@Override
	public void init(IConfiguration config) throws Exception {
		super.init(config);

		IConfiguration attr = config.getAttributes();
		String storeType = attr.get("storeType", null);
		if (storeType != null && storeType.length() > 0) {
			setStoreType(storeType);
		}
		String storePassword = attr.get("storePassword", null);
		if (storePassword != null && storePassword.length() > 0) {
			setStorePassword(storePassword);
		}
		String alias = attr.get("alias", null);
		if (alias != null && alias.length() > 0) {
			setAlias(alias);
		}
	}

	protected KeyStore getKeyStore()
		throws FileNotFoundException, IOException,
			KeyStoreException, NoSuchAlgorithmException, CertificateException
	{
		if (this.store != null) {
			return this.store;
		}
		InputStream is = null;
		try {
			if (getFile() != null) {
				is = new FileInputStream(getFile());
			} else {
				is = getInputStream();
			}
			KeyStore store = KeyStore.getInstance(getStoreType());
			char[] pwd = null;
			if (getStorePassword() != null) {
				pwd = getStorePassword().toCharArray();
			}
			store.load(is, pwd);
			this.store = store;
		} finally {
			if (is != null) {
				is.close();
			}
		}
		return store;
	}

    public String getStoreType() {
		return storeType;
	}

	@Override
	public void setStoreType(String storeType) {
		this.storeType = storeType;
	}

	public String getStorePassword() {
		return storePassword;
	}

	@Override
	public void setStorePassword(String storePassword) {
		this.storePassword = storePassword;
	}

	public String getAlias() {
		return alias;
	}

	@Override
	public void setAlias(String alias) {
		this.alias = alias;
	}

	@Override
	public KeySpec getKeySpec() throws Exception {
		throw(new UnsupportedOperationException("not supported"));
	}

}
