package jp.co.sds_corp.util.crypt;

import java.io.BufferedReader;
import java.io.InputStreamReader;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

public class CipherFactory
{
	public static ICipher createCipher(IConfiguration config) throws Exception {
		ICipher cipher = null;
		IConfiguration attr = config.getAttributes();
		String cls = attr.get("class");
		if (cls != null && cls.length() > 0) {
			cipher = (ICipher )Class.forName(cls).newInstance();
		}
		if (cipher == null) {
			throw(new IllegalArgumentException("class not defined."));
		}

		if (cipher instanceof IConfigurable) {
			((IConfigurable )cipher).init(config);
		}
		return cipher;
	}

	static void usage() {
		System.err.println("usage: " + CipherFactory.class.getSimpleName() + " [XML] [name]");
	}

	public static void main(String[] args) {
		try {
			if (args.length < 3) {
				usage();
				return;
			}
			DOMConfiguration doc = DOMConfiguration.findResource(args[0]);
			IConfiguration config = doc.getChild("cipher", "name", args[1]);
			boolean encrypt = true;
			if (args[2].equals("encrypt")) {
			} else if (args[2].equals("decrypt")) {
				encrypt = false;
			} else {
				throw(new IllegalArgumentException("encrypt or decrypt required"));
			}
			ICipher cipher = createCipher(config);
			StringBuilder buf = new StringBuilder();
			BufferedReader rd = new BufferedReader(new InputStreamReader(System.in));
			for ( ;; ) {
				String line = rd.readLine();
				if (line == null) {
					break;
				}
				buf.append(line).append("\n");
			}
			if (encrypt) {
				byte[] encrypted = cipher.encrypt(buf.toString().getBytes());
				System.out.print(new Base64Encoder().encode(encrypted));
			} else {
				byte[] encrypted = new Base64Encoder().decode(buf.toString());
				byte[] decrypted = cipher.decrypt(encrypted);
				System.out.print(new String(decrypted));
			}
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

}
