package	jp.co.sds_corp.util.db;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.CallableStatement;
import java.sql.Clob;
import java.sql.NClob;
import java.sql.Ref;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.util.Calendar;
import java.util.Map;
import java.util.HashMap;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* CallableStatementのラッパクラスです。
* SQLストアードプロシージャ実行時に、SQL文と動的パラメータを
* ログファイルに出力します。
* @author   小見山　聡
*/
public class DbCallableStatement extends DbPreparedStatement
	implements CallableStatement
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	CallableStatement stmt;
	Map<String,String> namedMap = new HashMap<String,String>();

	protected DbCallableStatement( CallableStatement stmt ) {
		super( stmt );
		this.stmt = stmt;
	}

	protected void addParameter( String name, String var ) {
		namedMap.put( name, var );
	}

	protected String parameterString() {
		String ret = super.parameterString();
		ret += namedMap.toString();
		return ret;
	}

    public void registerOutParameter(int parameterIndex, int sqlType) throws SQLException {
		stmt.registerOutParameter(parameterIndex, sqlType);
	}
    public void registerOutParameter(int parameterIndex, int sqlType, int scale) throws SQLException {
		stmt.registerOutParameter(parameterIndex, sqlType, scale);
	}
    public boolean wasNull() throws SQLException {
		return stmt.wasNull();
	}
    public String getString(int parameterIndex) throws SQLException {
		return stmt.getString(parameterIndex);
	}
    public boolean getBoolean(int parameterIndex) throws SQLException {
		return stmt.getBoolean(parameterIndex);
	}
    public byte getByte(int parameterIndex) throws SQLException {
		return stmt.getByte(parameterIndex);
	}
    public short getShort(int parameterIndex) throws SQLException {
		return stmt.getShort(parameterIndex);
	}
    public int getInt(int parameterIndex) throws SQLException {
		return stmt.getInt(parameterIndex);
	}
    public long getLong(int parameterIndex) throws SQLException {
		return stmt.getLong(parameterIndex);
	}
    public float getFloat(int parameterIndex) throws SQLException {
		return stmt.getFloat(parameterIndex);
	}
    public double getDouble(int parameterIndex) throws SQLException {
		return stmt.getDouble(parameterIndex);
	}
    public BigDecimal getBigDecimal(int parameterIndex, int scale) throws SQLException {
		return stmt.getBigDecimal(parameterIndex, scale);
	}
    public byte[] getBytes(int parameterIndex) throws SQLException {
		return stmt.getBytes(parameterIndex);
	}
    public java.sql.Date getDate(int parameterIndex) throws SQLException {
		return stmt.getDate(parameterIndex);
	}
    public java.sql.Time getTime(int parameterIndex) throws SQLException {
		return stmt.getTime(parameterIndex);
	}
    public java.sql.Timestamp getTimestamp(int parameterIndex) throws SQLException {
		return stmt.getTimestamp(parameterIndex);
	}

    //----------------------------------------------------------------------
    // Advanced features:

    public Object getObject(int parameterIndex) throws SQLException {
		return stmt.getObject(parameterIndex);
	}

    //--------------------------JDBC 2.0-----------------------------

    public BigDecimal getBigDecimal(int parameterIndex) throws SQLException {
		return stmt.getBigDecimal(parameterIndex);
	}
    public Object getObject(int parameterIndex, java.util.Map<String,Class<?>> map) throws SQLException {
		return stmt.getObject(parameterIndex, map);
	}
    public Ref getRef(int parameterIndex) throws SQLException {
		return stmt.getRef(parameterIndex);
	}
    public Blob getBlob(int parameterIndex) throws SQLException {
		return stmt.getBlob(parameterIndex);
	}
    public Clob getClob (int parameterIndex) throws SQLException {
		return stmt.getClob(parameterIndex);
	}
    public Array getArray(int parameterIndex) throws SQLException {
		return stmt.getArray(parameterIndex);
	}
    public java.sql.Date getDate(int parameterIndex, Calendar cal) throws SQLException {
		return stmt.getDate(parameterIndex, cal);
	}
    public java.sql.Time getTime(int parameterIndex, Calendar cal) throws SQLException {
		return stmt.getTime(parameterIndex, cal);
	}
    public java.sql.Timestamp getTimestamp(int parameterIndex, Calendar cal) throws SQLException {
		return stmt.getTimestamp(parameterIndex, cal);
	}
    public void registerOutParameter(int parameterIndex, int sqlType, String typeName) throws SQLException {
		stmt.registerOutParameter(parameterIndex, sqlType, typeName);
	}

	//--------------------------JDBC 3.0-----------------------------

    public void registerOutParameter(String parameterName, int sqlType) throws SQLException {
		stmt.registerOutParameter(parameterName, sqlType);
	}
    public void registerOutParameter(String parameterName, int sqlType, int scale) throws SQLException {
		stmt.registerOutParameter(parameterName, sqlType, scale);
	}
    public void registerOutParameter(String parameterName, int sqlType, String typeName) throws SQLException {
		stmt.registerOutParameter(parameterName, sqlType, typeName);
	}
    public java.net.URL getURL(int parameterIndex) throws SQLException {
		return stmt.getURL(parameterIndex);
	}
    public void setURL(String parameterName, java.net.URL val) throws SQLException {
		addParameter( parameterName, val == null ? "null" : val.toString() );
		stmt.setURL(parameterName, val);
	}
    public void setNull(String parameterName, int sqlType) throws SQLException {
		addParameter( parameterName, "null" );
		stmt.setNull(parameterName, sqlType);
	}
    public void setBoolean(String parameterName, boolean x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setBoolean(parameterName, x);
	}
    public void setByte(String parameterName, byte x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setByte(parameterName, x);
	}
    public void setShort(String parameterName, short x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setShort(parameterName, x);
	}
    public void setInt(String parameterName, int x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setInt(parameterName, x);
	}
    public void setLong(String parameterName, long x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setLong(parameterName, x);
	}
    public void setFloat(String parameterName, float x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setFloat(parameterName, x);
	}
    public void setDouble(String parameterName, double x) throws SQLException {
		addParameter( parameterName, String.valueOf( x ));
		stmt.setDouble(parameterName, x);
	}
    public void setBigDecimal(String parameterName, BigDecimal x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setBigDecimal(parameterName, x);
	}
    public void setString(String parameterName, String x) throws SQLException {
		addParameter( parameterName, x );
		stmt.setString(parameterName, x);
	}
    public void setBytes(String parameterName, byte x[]) throws SQLException {
		addParameter( parameterName, new String( x ));
		stmt.setBytes(parameterName, x);
	}
    public void setDate(String parameterName, java.sql.Date x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setDate(parameterName, x);
	}
    public void setTime(String parameterName, java.sql.Time x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setTime(parameterName, x);
	}
    public void setTimestamp(String parameterName, java.sql.Timestamp x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setTimestamp(parameterName, x);
	}
    public void setAsciiStream(String parameterName, java.io.InputStream x, int length) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setAsciiStream(parameterName, x, length);
	}
    public void setBinaryStream(String parameterName, java.io.InputStream x, int length) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setBinaryStream(parameterName, x, length);
	}
    public void setObject(String parameterName, Object x, int targetSqlType, int scale) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setObject(parameterName, x, targetSqlType, scale);
	}
    public void setObject(String parameterName, Object x, int targetSqlType) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setObject(parameterName, x, targetSqlType);
	}
    public void setObject(String parameterName, Object x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setObject(parameterName, x);
	}
    public void setCharacterStream(String parameterName, java.io.Reader reader, int length) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream(parameterName, reader, length);
	}
    public void setDate(String parameterName, java.sql.Date x, Calendar cal) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setDate(parameterName, x, cal);
	}
    public void setTime(String parameterName, java.sql.Time x, Calendar cal) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setTime(parameterName, x, cal);
	}
    public void setTimestamp(String parameterName, java.sql.Timestamp x, Calendar cal) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setTimestamp(parameterName, x, cal);
	}
    public void setNull (String parameterName, int sqlType, String typeName) throws SQLException {
		addParameter( parameterName, "null" );
		stmt.setNull (parameterName, sqlType, typeName);
	}
    public String getString(String parameterName) throws SQLException {
		return stmt.getString(parameterName);
	}
    public boolean getBoolean(String parameterName) throws SQLException {
		return stmt.getBoolean(parameterName);
	}
    public byte getByte(String parameterName) throws SQLException {
		return stmt.getByte(parameterName);
	}
    public short getShort(String parameterName) throws SQLException {
		return stmt.getShort(parameterName);
	}
    public int getInt(String parameterName) throws SQLException {
		return stmt.getInt(parameterName);
	}
    public long getLong(String parameterName) throws SQLException {
		return stmt.getLong(parameterName);
	}
    public float getFloat(String parameterName) throws SQLException {
		return stmt.getFloat(parameterName);
	}
    public double getDouble(String parameterName) throws SQLException {
		return stmt.getDouble(parameterName);
	}
    public byte[] getBytes(String parameterName) throws SQLException {
		return stmt.getBytes(parameterName);
	}
    public java.sql.Date getDate(String parameterName) throws SQLException {
		return stmt.getDate(parameterName);
	}
    public java.sql.Time getTime(String parameterName) throws SQLException {
		return stmt.getTime(parameterName);
	}
    public java.sql.Timestamp getTimestamp(String parameterName) throws SQLException {
		return stmt.getTimestamp(parameterName);
	}
    public Object getObject(String parameterName) throws SQLException {
		return stmt.getObject(parameterName);
	}
    public BigDecimal getBigDecimal(String parameterName) throws SQLException {
		return stmt.getBigDecimal(parameterName);
	}
    public Object getObject(String parameterName, java.util.Map<String,Class<?>> map) throws SQLException {
		return stmt.getObject(parameterName, map);
	}
    public Ref getRef(String parameterName) throws SQLException {
		return stmt.getRef(parameterName);
	}
    public Blob getBlob(String parameterName) throws SQLException {
		return stmt.getBlob(parameterName);
	}
    public Clob getClob(String parameterName) throws SQLException {
		return stmt.getClob(parameterName);
	}
    public Array getArray(String parameterName) throws SQLException {
		return stmt.getArray(parameterName);
	}
    public java.sql.Date getDate(String parameterName, Calendar cal) throws SQLException {
		return stmt.getDate(parameterName, cal);
	}
    public java.sql.Time getTime(String parameterName, Calendar cal) throws SQLException {
		return stmt.getTime(parameterName, cal);
	}
    public java.sql.Timestamp getTimestamp(String parameterName, Calendar cal) throws SQLException {
		return stmt.getTimestamp(parameterName, cal);
	}
    public java.net.URL getURL(String parameterName) throws SQLException {
		return stmt.getURL(parameterName);
	}

    //------------------------- JDBC 4.0 -----------------------------------

    public RowId getRowId(int parameterIndex) throws SQLException {
		return stmt.getRowId(parameterIndex);
	}
    public RowId getRowId(String parameterName) throws SQLException {
		return stmt.getRowId(parameterName);
	}
    public void setRowId(String parameterName, RowId x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setRowId(parameterName, x);
	}
    public void setNString(String parameterName, String value) throws SQLException {
		addParameter( parameterName, value );
		stmt.setNString(parameterName, value);
	}
    public void setNCharacterStream(String parameterName, Reader value, long length) throws SQLException {
		addParameter( parameterName, value == null ? "null" : value.toString() );
		stmt.setNCharacterStream(parameterName, value, length);
	}
    public void setNClob(String parameterName, NClob value) throws SQLException {
		addParameter( parameterName, value == null ? "null" : value.toString() );
		stmt.setNClob(parameterName, value);
	}
	public void setClob(String parameterName, Reader reader, long length) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setClob(parameterName, reader, length);
	}
    public void setBlob(String parameterName, InputStream inputStream, long length) throws SQLException {
		addParameter( parameterName, inputStream == null ? "null" : inputStream.toString() );
		stmt.setBlob(parameterName, inputStream, length);
	}
    public void setNClob(String parameterName, Reader reader, long length) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setNClob(parameterName, reader, length);
	}
    public NClob getNClob (int parameterIndex) throws SQLException {
		return stmt.getNClob (parameterIndex);
	}
    public NClob getNClob (String parameterName) throws SQLException {
		return stmt.getNClob (parameterName);
	}
    public void setSQLXML(String parameterName, SQLXML xmlObject) throws SQLException {
		stmt.setSQLXML(parameterName, xmlObject);
	}
    public SQLXML getSQLXML(int parameterIndex) throws SQLException {
		return stmt.getSQLXML(parameterIndex);
	}
    public SQLXML getSQLXML(String parameterName) throws SQLException {
		return stmt.getSQLXML(parameterName);
	}
    public String getNString(int parameterIndex) throws SQLException {
		return stmt.getNString(parameterIndex);
	}
    public String getNString(String parameterName) throws SQLException {
		return stmt.getNString(parameterName);
	}
    public java.io.Reader getNCharacterStream(int parameterIndex) throws SQLException {
		return stmt.getNCharacterStream(parameterIndex);
	}
    public java.io.Reader getNCharacterStream(String parameterName) throws SQLException {
		return stmt.getNCharacterStream(parameterName);
	}
    public java.io.Reader getCharacterStream(int parameterIndex) throws SQLException {
		return stmt.getCharacterStream(parameterIndex);
	}
    public java.io.Reader getCharacterStream(String parameterName) throws SQLException {
		return stmt.getCharacterStream(parameterName);
	}
    public void setBlob (String parameterName, Blob x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setBlob (parameterName, x);
	}
    public void setClob (String parameterName, Clob x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setClob (parameterName, x);
	}
    public void setAsciiStream(String parameterName, java.io.InputStream x, long length) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setAsciiStream(parameterName, x, length);
	}
    public void setBinaryStream(String parameterName, java.io.InputStream x, long length) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setBinaryStream(parameterName, x, length);
	}
    public void setCharacterStream(String parameterName, java.io.Reader reader, long length) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream(parameterName, reader, length);
	}
    //--
    public void setAsciiStream(String parameterName, java.io.InputStream x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setAsciiStream(parameterName, x);
	}
    public void setBinaryStream(String parameterName, java.io.InputStream x) throws SQLException {
		addParameter( parameterName, x == null ? "null" : x.toString() );
		stmt.setBinaryStream(parameterName, x);
	}
    public void setCharacterStream(String parameterName, java.io.Reader reader) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream(parameterName, reader);
	}
    public void setNCharacterStream(String parameterName, Reader value) throws SQLException {
		addParameter( parameterName, value == null ? "null" : value.toString() );
		stmt.setNCharacterStream(parameterName, value);
	}
    public void setClob(String parameterName, Reader reader) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setClob(parameterName, reader);
	}
    public void setBlob(String parameterName, InputStream inputStream) throws SQLException {
		addParameter( parameterName, inputStream == null ? "null" : inputStream.toString() );
		stmt.setBlob(parameterName, inputStream);
	}
    public void setNClob(String parameterName, Reader reader) throws SQLException {
		addParameter( parameterName, reader == null ? "null" : reader.toString() );
		stmt.setNClob(parameterName, reader);
	}

    //------------------------- Java7 -----------------------------------
	@Override
	public <T> T getObject(String parameterName, Class<T> type)
		throws SQLException
	{
		return stmt.getObject(parameterName, type);
	}

	@Override
	public <T> T getObject(int parameterIndex, Class<T> type)
		throws SQLException
	{
		return stmt.getObject(parameterIndex, type);
	}

}
