package	jp.co.sds_corp.util.db;

import java.io.PrintWriter;
import java.io.Serializable;
import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;
import java.text.DecimalFormat;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnDate;
import jp.co.sds_corp.util.common.CmnString;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* テーブル情報を保持します。
* @author 小見山　聡
*/
public class DbTable implements Serializable, Cloneable
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	/** カラム情報 */
	Vector<DbColumn> vcol = new Vector<DbColumn>();
	/** カラム数 */
	int			nCol = 0;
	/** カラムの検索情報 */
	HashMap<String,Integer>	hcol = new HashMap<String,Integer>();
	/** カラムがexecuteQueryで初期化されているか */
	boolean		bInit = false;
	/** テーブル名 */
	String	tbl = null;
	/** Audit カラム */
	Vector<String> listAudit = new Vector<String>();
	/** 日付フォーマット */
	String	dateFormat = "yyyy-MM-dd HH:mm:ss";

	/**
	* コンストラクタ
	* @param tbl	テーブル名
	*/
	public DbTable( String tbl ) {
		this();
		this.tbl = tbl;
	}
	/**
	* コンストラクタ
	*/
	public DbTable() {
	}

	/**
	* テーブル名を設定する
	* @param tbl	テーブル名
	*/
	public void setTableName( String tbl ) {
		this.tbl = tbl;
	}
	/**
	* テーブル名を取得する
	* @return		テーブル名
	*/
	public String getTableName() {
		return this.tbl;
	}

	/**
	* Auditカラムを設定する
	* @param var	Auditカラム
	*/
	protected void setAuditColumn( String var ) {
		listAudit.clear();
		listAudit.add( var );
	}
	/**
	* Auditカラムを取得する
	* @return		Auditカラム
	*/
	public String getAuditColumn() {
		return (String )listAudit.get( 0 );
	}
	/**
	* Auditカラムを設定する
	* @param var	Auditカラム
	*/
	protected void addAuditColumn( String var ) {
		listAudit.add( var );
	}
	/**
	* Auditカラムを取得する
	* @return		Auditカラム
	*/
	public List<String> listAuditColumn() {
		return this.listAudit;
	}

	/**
	* Auditカラムであるかどうか
	* @return		Auditカラム
	*/
	public boolean isAuditColumn( String var ) {
		return ( listAudit.indexOf( var ) >= 0 );
	}

	/**
	* カラム情報の取得
	* @param rset	結果セット
	*/
	protected void setColumnInfo( ResultSet rset ) throws SQLException {
		try {
			clearColumn();
			ResultSetMetaData rsmd = rset.getMetaData();
			int nCol = rsmd.getColumnCount();
			for ( int nLoop = 0; nLoop < nCol; nLoop++ ) {
				addColumn( new DbColumn( rsmd, nLoop+1 ));
			}
			bInit = true;
		} catch ( SQLException ex ) {
			throw ex;
		}
	}

	/**
	* カラム情報の初期化
	*/
	public void clearColumn() {
		nCol = 0;
		bInit = false;
		vcol.clear();
		hcol.clear();
	}

	/**
	* カラムの初期化情報の設定
	* @param	flg		初期化フラグ
	*/
	public void setInitColumn( boolean flg ) {
		bInit = flg;
	}
	/**
	* カラムが初期化されているか
	* @return		初期化情報
	*/
	public boolean isInitColumn() {
		return bInit;
	}

	/**
	* カラム情報の設定
	* @param col	カラム情報
	*/
	public void addColumn( DbColumn col ) {
		vcol.add( col );
		hcol.put( col.getName(), Integer.valueOf( nCol ));
		nCol++;
	}

	/**
	* カラム数の取得
	* @return		カラム数
	*/
	public int getColumnCount() {
		return nCol;
	}

	/**
	* カラム情報の取得
	* @param nLoop	カラム番号(1〜)
	* @return		カラム情報
	*/
	public DbColumn getColumn( int nLoop ) {
		return (DbColumn )vcol.get( nLoop );
	}

	/**
	* カラム内容の取得
	* @param nLoop	カラム番号(1〜)
	* @return		カラム内容（テーブルの型にあったオブジェクトで取得される）
	*/
	public Object getObject( int nLoop ) {
		return ((DbColumn )vcol.get( nLoop )).getObject();
	}

	/**
	* カラム内容の取得
	* @param nLoop	カラム番号(1〜)
	* @return		文字列表現のカラム内容
	*/
	public String getString( int nLoop ) {
		Object obj = ((DbColumn )vcol.get( nLoop )).getObject();
		if ( obj == null ) {
			return null;
		}
		return obj.toString();
	}

	/**
	* カラム内容の設定
	* @param nLoop	カラム番号(1〜)
	* @param obj	設定するObject
	*/
	public void setObject( int nLoop, Object obj ) {
		((DbColumn )vcol.get( nLoop )).setObject( obj );
	}

	/**
	* カラム番号の取得
	* @param nam	カラム名
	* @return		カラム内容（テーブルの型にあったオブジェクトで取得される）
	*/
	public int getIndex( String nam ) {
		Integer pos = (Integer )hcol.get( nam );
		if ( pos == null ) {
			return -1;
		}
		return pos.intValue();
	}

	/**
	* カラム情報の取得
	* @param nam	カラム名
	* @return		カラム情報
	*/
	public DbColumn getColumn( String nam ) {
		int		nCol = getIndex( nam );
		if ( nCol < 0 ) {
			return null;
		}
		return (DbColumn )vcol.get( nCol );
	}

	/**
	* カラム内容の取得
	* @param nam	カラム名
	* @return		カラム内容（テーブルの型にあったオブジェクトで取得される）
	*/
	public Object getObject( String nam ) {
		int		nCol = getIndex( nam );
		if ( nCol < 0 ) {
			return null;
		}
		return ((DbColumn )vcol.get( nCol )).getObject();
	}

	/**
	* カラム内容の取得
	* @param nam	カラム名
	* @return		文字列表現のカラム内容
	*/
	public String getString( String nam ) {
		int		nCol = getIndex( nam );
		if ( nCol < 0 ) {
			return null;
		}
		Object obj = ((DbColumn )vcol.get( nCol )).getObject();
		if ( obj == null ) {
			return null;
		}
		return obj.toString();
	}

	/**
	* カラム内容の設定
	* @param nam	カラム名
	* @param obj	設定するObject
	*/
	public void setObject( String nam, Object obj ) {
		int		nCol = getIndex( nam );
		if ( nCol < 0 ) {
			return;
		}
		((DbColumn )vcol.get( nCol )).setObject( obj );
	}

	/**
	* 日付フォーマットを設定します。
	* @param	var		設定するデータ
	*/
	public void setDateFormat( String var ) {
		this.dateFormat = var;
	}
	/**
	* 日付フォーマットを取得します。
	* @return	日付フォーマット
	*/
	public String getDateFormat() {
		return dateFormat;
	}

	/**
	* 日付カラムの編集
	* @param nam	カラム名
	* @param fmt	フォーマット文字列
	* @return		文字列表現のカラム内容
	* @exception	Exception	日付編集例外
	*/
	public String formatDate( String nam, String fmt )
		throws java.text.ParseException
	{
		Object obj = getObject( nam );
		if ( obj == null ) {
			return null;
		}
		CmnDate dt = null;
		if ( obj instanceof Timestamp ) {
			dt = new CmnDate( (java.util.Date )obj );
		} else {
/*
			throw new IllegalArgumentException(
						"not Timestamp column ["+nam+"]" );
*/
			dt = new CmnDate( getString( nam ));
		}
		return dt.format( fmt );
	}

	/**
	* Decimalカラムの編集
	* @param nam	カラム名
	* @param fmt	フォーマット文字列
	* @return		文字列表現のカラム内容
	* @exception	Exception	Decimal編集例外
	*/
	public String formatDecimal( String nam, String fmt ) {
		DecimalFormat df = new DecimalFormat( fmt );
		Object obj = getObject( nam );
		if ( obj == null ) {
			return null;
		}
		if (!( obj instanceof Number )) {
			throw new IllegalArgumentException( "not Number column ["
				+ nam + ":" + obj.getClass().getName() + "]" );
		}
		return df.format( ((Number )getObject( nam )).doubleValue() );
	}

	/**
	* 文字列を解析して、テーブルのデータに変換します。
	* @param	line		入力文字列
	*/
	public void parse( String line ) throws Exception {
		String[] arr = CmnString.getTokens( line, "\t" );
		parse( arr );
	}

	/**
	* 文字列を解析して、テーブルのデータに変換します。
	* @param	arr			入力データ
	*/
	public void parse( String[] arr ) throws Exception {
		if ( arr.length != getColumnCount()) {
			throw( new IllegalArgumentException( "column count=" + arr.length
					+ " must be " + getColumnCount() ));
		}
		for ( int i = 0; i < arr.length; i++ ) {
			try {
				switch ( getColumn( i ).getType()) {
				case Types.VARCHAR:
				case Types.CHAR:
				case Types.LONGVARCHAR:
					setObject( i, arr[i] );
					break;
				case Types.BIT:
				case Types.TINYINT: 
				case Types.BIGINT:
				case Types.INTEGER:
				case Types.SMALLINT:
					if ( arr[i] != null && arr[i].length() > 0 ) {
						setObject( i, Integer.valueOf( arr[i] ));
					} else {
						setObject( i, null );
					}
					break;
				case Types.NUMERIC:
				case Types.DECIMAL:
					if ( arr[i] != null && arr[i].length() > 0 ) {
						BigDecimal obj = new BigDecimal( arr[i] );
						if ( getColumn( i ).getScale() >= 0 ) {
							obj = obj.setScale( getColumn( i ).getScale(),
												BigDecimal.ROUND_HALF_UP );
						}
						setObject( i, obj );
					} else {
						setObject( i, null );
					}
					break;
				case Types.REAL:
				case Types.FLOAT:
					if ( arr[i] != null && arr[i].length() > 0 ) {
						setObject( i, new Float( arr[i] ));
					} else {
						setObject( i, null );
					}
					break;
				case Types.DOUBLE:
					if ( arr[i] != null && arr[i].length() > 0 ) {
						setObject( i, new Double( arr[i] ));
					} else {
						setObject( i, null );
					}
					break;
				case Types.DATE:
				case Types.TIMESTAMP:
					CmnDate date = null;
					try {
						date = new CmnDate( arr[i], dateFormat );
					} catch ( Exception e1 ) {
						try {
							date = new CmnDate( arr[i],
									"MMM d yyyy HH:mm:ss:SSSaa", Locale.US );
						} catch ( Exception e2 ) {
						}
					}
					if ( date != null ) {
						setObject( i, date.getTimestamp() );
					} else {
						setObject( i, null );
					}
					break;
				default:
					throw( new DbException( "type error:" + getColumn( i ).getType() ));
				}
			} catch ( Throwable ex ) {
				DbException dbe = new DbException( "column["
					+ getColumn( i ).getName() + "] error", ex );
				throw( dbe );
			}
		}
	}

	/**
	* テーブルの内容を出力します。
	* @param	out		出力先のPrintWriter
	*/
	public void putBcp( PrintWriter out ) {
		for ( int i = 0; i < getColumnCount(); i++ ) {
			if ( i > 0 ) {
				out.print( "\t" );
			}
			DbColumn col = getColumn( i );
			String dat;
			if ( isAuditColumn( col.getName()) ) {
				if ( getObject( i ) == null ) {
					dat = new CmnDate().getString();
				} else {
					dat = new CmnDate( (Timestamp )getObject( i ))
							.format( "yyyy-MM-dd HH:mm:ss" );
				}
			} else {
				dat = getString( i );
			}
			if ( dat == null || dat.length() == 0 ) {
				dat = " ";
			}
			switch ( col.getType()) {
			case Types.VARCHAR:
			case Types.CHAR:
			case Types.LONGVARCHAR:
				dat = CmnString.replaceAll( dat, "\t", "" );
				dat = CmnString.replaceAll( dat, "\n", "" );
				if ( dat.length() > col.getSize() && col.getSize() > 0 ) {
					logger.debug( "truncate string"
						+ " " + getTableName() + "." + col.getName()
						+ "(" + col.getSize() + " ) "
						+ "[" + dat + "]" );
					dat = dat.substring( 0, col.getSize() );
				}
			}
			out.print( dat );
		}
		out.print( "\n" );
	}

	/**
	* 引数のテーブル情報からデータをコピーします。
	* @param	tbl		コピー元
	*/
	public void copyAllColumn( DbTable tbl ) {
		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			DbColumn col = tbl.getColumn( i );
			int nIdx = getIndex( col.getName() );
			if ( nIdx >= 0 ) {
				DbColumn col2 = getColumn( nIdx );
				switch ( col.getType()) {
				case Types.DOUBLE:
				case Types.FLOAT:
					if ( col2.getType() == Types.DECIMAL ||
						col2.getType() == Types.NUMERIC )
					{
						BigDecimal obj = new BigDecimal(
							((Number )tbl.getObject( i )).doubleValue());
						if ( col2.getScale() >= 0 ) {
							obj = obj.setScale( col2.getScale(), BigDecimal.ROUND_HALF_UP );
						}
						setObject( nIdx, obj );
					} else {
						setObject( nIdx, tbl.getObject( i ));
					}
					break;
				case Types.VARCHAR:
				case Types.CHAR:
				case Types.LONGVARCHAR:
				default:
					setObject( nIdx, tbl.getObject( i ));
					break;
				}
			}
		}
	}

	/**
	* Cloneable をオーバーライドします。
	* @return このインスタンスの複製
	* @exception CloneNotSupportedException オブジェクトのクラスが Cloneable インタフェースをサポートしていない場合。clone メソッドをオーバーライドしたサブクラスも、インスタンスを複製できないことを示すためにこの例外をスローすることがある
	* @exception OutOfMemoryError - 十分なメモリがない場合
	*/
	public Object clone() throws CloneNotSupportedException, OutOfMemoryError {
		DbTable tbl = (DbTable )super.clone();
		tbl.vcol = new Vector<DbColumn>();
		for ( Enumeration<DbColumn> en = vcol.elements(); en.hasMoreElements(); )
		{
			DbColumn col = (DbColumn )en.nextElement();
			tbl.vcol.add( (DbColumn )col.clone() );
		}
		return tbl;
	}

	public String getSelectSql( String where ) {
		return null;
	}
	public String getSelectByKeySql() {
		return null;
	}
	public String getInsertSql() {
		return null;
	}
	public String getDeleteSql() {
		return null;
	}

}
