package	jp.co.sds_corp.util.db;

import java.io.UnsupportedEncodingException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;

/**
* パスワードの暗号化、復号化を行う。
* @author 小見山　聡
*/
public class PasswordEncrypt
{
	// 暗号化アルゴリズム
	static final String algorithm = "Blowfish";
	static final String DEFAULT_KEY = "jp.co.sds_corp.util.db";

	/**
	* パスワードを暗号化する。
	* @param	passwd	パスワード
	*/
	public static String encrypt( String passwd )
		throws IllegalBlockSizeException,
			BadPaddingException, NoSuchPaddingException,
			InvalidKeyException, NoSuchAlgorithmException,
			UnsupportedEncodingException
	{
		return encrypt( DEFAULT_KEY, passwd );
	}
	/**
	* パスワードを復号化する。
	* @param	passwd	パスワード
	*/
	public static String decrypt( String passwd )
		throws IllegalBlockSizeException,
			BadPaddingException, NoSuchPaddingException,
			InvalidKeyException, NoSuchAlgorithmException,
			UnsupportedEncodingException
	{
		return decrypt( DEFAULT_KEY, passwd );
	}

	/**
	* パスワードを暗号化する。
	* @param	key		マスクするキー
	* @param	passwd	パスワード
	*/
	public static String encrypt( String key, String passwd )
		throws IllegalBlockSizeException,
			BadPaddingException, NoSuchPaddingException,
			InvalidKeyException, NoSuchAlgorithmException,
			UnsupportedEncodingException
	{
		MessageDigest md = MessageDigest.getInstance( "MD5" );
		byte[] bkey = md.digest( key.getBytes());

		SecretKeySpec spec = new SecretKeySpec( bkey, algorithm );
		Cipher cipher = Cipher.getInstance( algorithm );
		cipher.init( Cipher.ENCRYPT_MODE, spec );
		byte[] encrypted = cipher.doFinal( passwd.getBytes() );
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < encrypted.length; i++ ) {
			buf.append( Integer.toString(( encrypted[i] & 0xf0 ) >> 4, 16 ));
			buf.append( Integer.toString( encrypted[i] & 0x0f, 16 ));
		}
		return buf.toString();
	}

	/**
	* パスワードを復号化する。
	* @param	key		マスクするキー
	* @param	passwd	パスワード
	*/
	public static String decrypt( String key, String passwd )
		throws IllegalBlockSizeException,
			BadPaddingException, NoSuchPaddingException,
			InvalidKeyException, NoSuchAlgorithmException,
			UnsupportedEncodingException
	{
		MessageDigest md = MessageDigest.getInstance( "MD5" );
		byte[] bkey = md.digest( key.getBytes());

		SecretKeySpec spec = new SecretKeySpec( bkey, algorithm );
		Cipher cipher = Cipher.getInstance( algorithm );
		cipher.init( Cipher.DECRYPT_MODE, spec );
		byte[] buf = new byte[passwd.length() / 2];
		for ( int i = 0; i < passwd.length(); i += 2 ) {
			String s = passwd.substring( i, i + 2 );
			try {
				Integer val = Integer.parseInt( s, 16 );
				buf[i/2] = val.byteValue();
			} catch ( Exception ex ) {
				throw( new PasswordFormatException() );
			}
		}
		byte[] decrypted = cipher.doFinal( buf );
		return new String( decrypted );
	}

	public static void main( String[] args ) {
		if ( args.length != 1 && args.length != 2 ) {
			System.err.println( "usage: PasswordEncrypt {key} [text]" );
			System.exit( 9 );
		}

		try {
			String mask = null;
			if ( args.length == 1 ) {
				String pwd = args[0];
				mask = PasswordEncrypt.encrypt( pwd );
			} else {
				String key = args[0];
				String pwd = args[1];
				mask = PasswordEncrypt.encrypt( key, pwd );
			}
			System.out.println( mask );
/*
			System.out.println( "key=" + key + " pwd=" + pwd + "->" + mask );
			System.out.println( "decode=" + PasswordEncrypt.decrypt( key, mask ));
*/
		} catch ( Exception ex ) {
			ex.printStackTrace();
			System.exit( 9 );
		}
		System.exit( 0 );
	}

}
