package jp.co.sds_corp.util.diag;

import java.net.InterfaceAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.ArrayList;
import java.util.List;

import jp.co.sds_corp.util.log.ConsoleLogger;

public class NetworkDiagnostics
{
	ConsoleLogger logger = ConsoleLogger.getLogger(this);

	public void displayNetworks() {
		List<String> list = getNetworks();
		for (String msg : list) {
			logger.info(msg);
		}
	}
	public List<String> getNetworks() {
		ArrayList<String> ret = new ArrayList<String>();
		ret.add("=== network ===");
		try {
			Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
			while (en.hasMoreElements()) {
				NetworkInterface net = en.nextElement();
				ret.addAll(getNetworks(net, 0));
			}
		} catch (SocketException ex) {
			logger.error("network error", ex);
		}
		return ret;
	}

	List<String> getNetworks(NetworkInterface net, int indent)
		throws SocketException
	{
		ArrayList<String> ret = new ArrayList<String>();
		ret.add(indent(indent)
				+ net.getName() + "," + net.getDisplayName() + ": "
				+ net.getMTU()
				+ "<"
				+ (net.isUp() ? "up" : "down")
				+ (net.isLoopback() ? ",loopback" : "")
				+ (net.isPointToPoint() ? ",P2P" : "")
				+ (net.supportsMulticast() ? ",multicast" : "")
				+ (net.isVirtual() ? ",virtual" : "")
				+ ">");
		List<InterfaceAddress> addrs = net.getInterfaceAddresses();
		for (InterfaceAddress addr : addrs) {
			ret.add(indent(indent + 1) + addr.toString());
		}

		Enumeration<NetworkInterface> enSub = net.getSubInterfaces();
		while (enSub.hasMoreElements()) {
			NetworkInterface sub = enSub.nextElement();
			ret.addAll(getNetworks(sub, indent + 1));
		}
		return ret;
	}

	String indent(int indent) {
		StringBuilder buf = new StringBuilder();
		for (int i = 0; i < indent; i++) {
			buf.append("\t");
		}
		return buf.toString();
	}

}
