package jp.co.sds_corp.util.diag;

import java.applet.Applet;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.net.CookieManager;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.TreeSet;

import jp.co.sds_corp.util.log.ConsoleLogger;

public class SystemDiagnostics
{
	ConsoleLogger logger = ConsoleLogger.getLogger(this);
	ThreadMonitor threadMon = new ThreadMonitor();
	AppletDiagnostics appletDiag = new AppletDiagnostics();
	FileDiagnostics fileDiag = new FileDiagnostics();
	NetworkDiagnostics netDiag = new NetworkDiagnostics();
	MemoryDiagnostics memoryDiag = new MemoryDiagnostics();

	public void displaySystemInfo() {
		logger.info("ClassLoader=" + getClass().getClassLoader());
		List<String> list = new ArrayList<String>();
		list.addAll(getSystemProperties());
		list.addAll(memoryDiag.getMemoryInfo());
		list.addAll(getGraphicsEnvironment());
		list.addAll(fileDiag.getFilesystems());
//		list.addAll(netDiag.getNetworks());
		for (String msg : list) {
			logger.info(msg);
		}
	}

	public List<String> getSystemProperties() {
		ArrayList<String> ret = new ArrayList<String>();
		Properties sysProp = System.getProperties();
		Set<String> names = sysProp.stringPropertyNames();
		TreeSet<String> keys = new TreeSet<String>(names);

		ret.add("=== properties ===");
		String username = System.getProperty("user.name");
		String usershort = username.toUpperCase();
		if (usershort.length() > 8) {
			usershort = usershort.substring(0, 6);
		}
		for (String key : keys) {
			String value = sysProp.getProperty(key);
			if (value != null) {
				value = value.replace("\r", "\\r");
				value = value.replace("\n", "\\n");
				value = value.replace(username, "********");
				value = value.replace(usershort, "******");
			}
			ret.add(key + "=" + value);
		}
		return ret;
	}

	public List<String> getOSInfo() {
		ArrayList<String> ret = new ArrayList<String>();
		OperatingSystemMXBean osBean = ManagementFactory.getOperatingSystemMXBean();
		ret.add(osBean.getName() + " ver " + osBean.getVersion()
				+ " (" + osBean.getArch() + ")"
				+ " processors=" + osBean.getAvailableProcessors());
		return ret;
	}

	public List<String> getLoadAverage() {
		ArrayList<String> ret = new ArrayList<String>();
		OperatingSystemMXBean osBean = ManagementFactory.getOperatingSystemMXBean();
		ret.add("loadavg=" + osBean.getSystemLoadAverage()
				+ " processors=" + osBean.getAvailableProcessors());
		return ret;
	}

	public void displayMemoryInfo() {
		memoryDiag.displayMemoryInfo();
	}

	public List<String> getGraphicsEnvironment() {
		ArrayList<String> ret = new ArrayList<String>();
		ret.add("=== graphics env ===");
		Toolkit toolkit = Toolkit.getDefaultToolkit();
		Dimension size = toolkit.getScreenSize();
		ret.add("size=[" + size.width + "," + size.height + "]"
				+ "," + toolkit.getScreenResolution() + "dpi"
				+ ",alwaysOnTop=" + toolkit.isAlwaysOnTopSupported());

		ret.addAll(getFontList());
		return ret;
	}

	List<String> getGraphicsDevice() {
		ArrayList<String> ret = new ArrayList<String>();
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		for (GraphicsDevice device : env.getScreenDevices()) {
			ret.addAll(getGraphicsDevice(device));
		}
		return ret;
	}

	List<String> getGraphicsDevice(GraphicsDevice device) {
		ArrayList<String> ret = new ArrayList<String>();
		GraphicsConfiguration config = device.getDefaultConfiguration();
		Rectangle size = config.getBounds();
		ret.add(device.getIDstring()
				+ ",size[" + size.width + "," + size.height + "]"
				+ "," + config.getColorModel()
				+ "," + config.getDefaultTransform());
		return ret;
	}

	public List<String> getFontList() {
		ArrayList<String> ret = new ArrayList<String>();
		GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
		ret.add("=== font list ===");
		String[] fonts = env.getAvailableFontFamilyNames();
		StringBuilder log = new StringBuilder();
		for (String font : fonts) {
			log.append(font + ",");
		}
		ret.add(log.toString());
		return ret;
	}

	public void displayThread() {
		displayThread(false);
	}

	public void displayThread(boolean showStack) {
		threadMon.setShowStack(showStack);
		threadMon.displayThreadList();
	}

	public void displayApplet(Applet applet) {
		appletDiag.displayApplet(applet);
	}

	public void displayCookie(CookieManager manager) {
		appletDiag.displayCookie(manager);
	}
	
	public void displayNetworks() {
		netDiag.displayNetworks();
	}

}
