package	jp.co.sds_corp.util.log;

import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
* ログを出力します。
* @author	小見山　聡
*/
public class CmnLogFactory
{
	private static final String LOG_PROPERTIES = "log.properties";
	private static CmnLogFactory instance = new CmnLogFactory();
	private ConsoleLogger console = ConsoleLogger.getLogger(this);
	private LogLoader	loader;
	private boolean initialized = false;
	private Logger  rootLogger;
	private boolean needCaller = false;

	private CmnLogFactory() {
	}

	public LogLoader getLogLoader() {
		return loader;
	}

	void loadConfig() {
		try {
			loader = new LogLoader();
			loader.init();
		} catch ( Exception ex ) {
			console.error( "log config load error.", ex );
		}
		rootLogger = getJavaLogger( loader.getRoot() );
		needCaller = rootLogger.isLoggable( Level.FINER );
	}

	void loadProperty() {
		try {
			console.info( "load " + LOG_PROPERTIES );
			LogManager manager = LogManager.getLogManager();
			InputStream is = getClass().getClassLoader()
						.getResourceAsStream( LOG_PROPERTIES );
			if ( is == null ) {
				console.error( LOG_PROPERTIES + " not found." );
				return;
			}
			manager.readConfiguration( is );
		} catch ( IOException ex ) {
			console.error( LOG_PROPERTIES + " load error. ", ex );
		}
	}

	public static CmnLogFactory getInstance() {
		if ( !instance.initialized ) {
/*
			instance.loadProperty();
*/
			instance.loadConfig();
			instance.initialized = true;
		}
		return instance;
	}

	Logger getJavaLogger( Object obj ) {
		Logger ret = null;
		if ( obj instanceof Class ) {
			ret = getJavaLogger( ((Class<?> )obj).getName() );
		} else {
			ret = getJavaLogger( obj.getClass().getName() );
		} 
		return ret;
	}

	Logger getJavaLogger( String category ) {
		Logger ret = null;
		try {
			ret = Logger.getLogger( category );
		} catch ( NoClassDefFoundError e1 ) {
		} catch ( Exception e2 ) {
		}
		return ret;
	}

	public ConsoleLogger getConsoleLogger( Object obj ) {
		ConsoleLogger logger = new ConsoleLogger(obj);
		return logger;
	}

	public CmnLogger getLogger( Object obj ) {
		Logger logger = getJavaLogger(obj);
		return new CmnLogger(logger);
	}

	public CmnLogger getLogger( String category ) {
		Logger logger = getJavaLogger(category);
		return new CmnLogger(logger);
	}

	public boolean needCaller() {
		return needCaller;
	}

}
