package	jp.co.sds_corp.util.log;

import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;
import java.util.logging.SimpleFormatter;

import jp.co.sds_corp.util.config.IConfigurable;
import jp.co.sds_corp.util.config.IConfiguration;

/**
java.util.Formatterを使用して、printf形式のフォーマットで、ログを編集します。
パラメータは、
1:時間、2:レベル名、3:Logger名、4:Logger名（最終トークン）、5:スレッドID、
6:シーケンス番号、7:クラス名、8:メソッド名、9:メッセージ、10:例外
11:時間(yyyy/MM/dd HH:mm:ss.SSS)
の順に渡されますので、それに対応したフォーマットを指定しなくてはなりません。
*/
public class FormattableFormatter extends SimpleFormatter
	implements IConfigurable
{
	ConsoleLogger console = ConsoleLogger.getLogger(this);
	String format;
	SimpleDateFormat sdf1 = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss.SSS" );

	public FormattableFormatter() {
		super();
		LogManager manager = LogManager.getLogManager();
		format = manager.getProperty( getClass().getName() + ".format" );
		// console.log( "format=" + format );
	}

	public void init( IConfiguration config ) throws Exception {
		Iterator<IConfiguration> itr = config.getChilds( "param" );
		while ( itr.hasNext()) {
			IConfiguration ele = itr.next();
			String name = ele.getAttributes().get( "name" );
			String value = ele.getAttributes().get( "value" );
			if ( name == null ) {
			} else
			if ( name.equals( "format" )) {
				this.format = value;
			} else {
				console.error( "unknown param name=" + name );
			}
		}
		if ( format == null ) {
			console.error( "format is NULL" );
		}
	}

	public void setFormat( String format ) {
		this.format = format;
		// console.log( "set format=" + format );
	}

	public String format( LogRecord record ) {
		if ( format == null ) {
			// return super.format( record );
			return null;
		}
		String ret = "";
		try {
			String name = record.getLoggerName();
			String shortName = name;
			int idx = name.lastIndexOf( "." );
			if ( idx > 0 ) {
				shortName = name.substring( idx + 1 );
			}
			String clazz = "";
			String method = "";
			if ( format.indexOf( "%8" ) >= 0 ) {
				clazz = record.getSourceClassName();
				method = record.getSourceMethodName();
			}
			long time = record.getMillis();
			ret = String.format( format,
				time,
				record.getLevel().getName(),
				record.getLoggerName(),
				shortName,
				record.getThreadID(),
				record.getSequenceNumber(),
				clazz,
				method,
				record.getMessage(),
				record.getThrown(),
				sdf1.format( time ));
		} catch ( Exception ex ) {
			ConsoleLogger console = ConsoleLogger.getLogger(this);
			console.error( "format error.", ex );
		}
		return ret;
	}

}

