package	jp.co.sds_corp.util.mail;

import java.util.Properties;
import java.util.Vector;

import javax.mail.Message;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeUtility;

/**
* メール配信クラス。
* @author	小見山　聡
*/
public class MailSender
{
	/**
	* コンストラクタ
	*/
	public MailSender() {
	}

	String	host = null;
	String	from = null;
	Vector<String>	vecTo = new Vector<String>();
	Vector<String>	vecCc = new Vector<String>();
	Vector<String>	vecBcc = new Vector<String>();
	String	subject = null;
	String	msgTxt = null;

	/**
	* SMTPサーバを設定します。
	* @param	host	SMTPサーバ
	*/
	public void setHost( String host ) {
		this.host = host;
	}
	/**
	* 送信元のメールアドレスを設定します。
	* @param	from	送信元のメールアドレス
	*/
	public void setFrom( String from ) {
		this.from = from;
	}
	/**
	* メールのタイトルを設定します。
	* @param	subject	メールのタイトル
	*/
	public void setSubject( String subject ) {
		this.subject = subject;
	}
	/**
	* メールの本文を設定します。
	* @param	msg		メールの本文
	*/
	public void setMessage( String msg ) {
		this.msgTxt = msg;
	}
	/**
	* メールの送信先を追加します。
	* @param	to		メールの送信先
	*/
	public void addTo( String to ) {
		vecTo.add( to );
	}
	/**
	* メールのCC先を追加します。
	* @param	cc		メールのCC先
	*/
	public void addCc( String cc ) {
		vecCc.add( cc );
	}
	/**
	* メールのBCC先を追加します。
	* @param	bcc		メールのBCC先
	*/
	public void addBcc( String bcc ) {
		vecBcc.add( bcc );
	}

	/**
	* メール送信
	*/
	public void send() throws Exception {
		Properties props = System.getProperties();
		props.put( "mail.smtp.host", host );
//		props.put( "mail.debug", "true" );
		Session session = Session.getDefaultInstance( props, null );
		session.setDebug( false );

		MimeMessage msg = new MimeMessage( session );
		msg.setFrom( new InternetAddress( from ));
		InternetAddress[] address = new InternetAddress[vecTo.size()];
		for ( int nLoop = 0; nLoop < vecTo.size(); nLoop++ ) {
			address[nLoop] = new InternetAddress( (String )vecTo.get( nLoop ));
		}
		msg.setRecipients( Message.RecipientType.TO, address );
		if ( vecCc.size() > 0 ) {
			InternetAddress[] address_cc = new InternetAddress[vecCc.size()];
			for ( int nLoop = 0; nLoop < vecCc.size(); nLoop++ ) {
				address_cc[nLoop] = new InternetAddress( (String )vecCc.get( nLoop ));
			}
			msg.setRecipients( Message.RecipientType.CC, address_cc );
		}
		if ( vecBcc.size() > 0 ) {
			InternetAddress[] address_bcc = new InternetAddress[vecBcc.size()];
			for ( int nLoop = 0; nLoop < vecBcc.size(); nLoop++ ) {
				address_bcc[nLoop] = new InternetAddress( (String )vecBcc.get( nLoop ));
			}
			msg.setRecipients( Message.RecipientType.BCC, address_bcc );
		}
		msg.setSubject( MimeUtility.encodeWord( subject, "ISO-2022-JP", "B" ));
		msg.setText( msgTxt, "ISO-2022-JP" );
		msg.setSentDate( new java.util.Date());
		Transport.send( msg );
	}

}
